/**
  *  \file client/tiles/baseoverview.cc
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/tiles/baseoverview.h"
#include "ui/uiglobal.h"
#include "game/planet.h"
#include "game/univ.h"
#include "util/nls.h"
#include "cpluslib/format.h"
#include "game/ship.h"
#include "util/misc.h"

WBaseOverviewTile::WBaseOverviewTile(GObjectSelection& selection)
    : WObjectSelectionObserverTile(font_sizes[FONT_NORMAL_BOLD].scaledBy(30, 10), selection)
{ }

void
WBaseOverviewTile::drawData(GfxCanvas& can)
{
    GPlanet* p = dynamic_cast<GPlanet*>(getCurrentObject());
    if (!p)
        return;

    GPlanet& pp = *p;

    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(getSkin());
    ctx.useFont(*fonts[FONT_NORMAL]);
    ctx.setColor(UIColor::tc_Static);
    ctx.setSolidBackground();

    int y = 0, x = 0;
    const int h = font_heights[FONT_NORMAL];

    outTextF(ctx, x, y, extent.w, format(_("%d mc; %d kt supplies"),
                                         numToString(pp.getCargo(el_Money)),
                                         numToString(pp.getCargo(el_Supplies))));
    y += h;

    /* minerals */
    outTextF(ctx, x, y, 70, _("Minerals:"));
    ctx.setTextAlign(2, 0);
    outTextF(ctx, x+130, y, 60, format("%dN", numToString(pp.getCargo(el_Neutronium))));
    outTextF(ctx, x+210, y, 80, format("%dT", numToString(pp.getCargo(el_Tritanium))));
    outTextF(ctx, x+290, y, 80, format("%dD", numToString(pp.getCargo(el_Duranium))));
    outTextF(ctx, x+370, y, 80, format("%dM", numToString(pp.getCargo(el_Molybdenum))));
    y += h;

    /* tech */
    ctx.setTextAlign(0, 0);
    outTextF(ctx, x, y, 100, _("Tech Levels:"));
    ctx.setTextAlign(2, 0);
    outTextF(ctx, x+130, y, 30, format("%dH", pp.getBaseTechLevel(tl_Hull)));
    outTextF(ctx, x+210, y, 80, format("%dE", pp.getBaseTechLevel(tl_Engine)));
    outTextF(ctx, x+290, y, 80, format("%dB", pp.getBaseTechLevel(tl_Beam)));
    outTextF(ctx, x+370, y, 80, format("%dT", pp.getBaseTechLevel(tl_Torpedo)));
    y += h;
    y += h/3;

    /* are we building something? */
    ctx.setTextAlign(0, 0);
    if (int sid = pp.findShipCloningHere(*getCurrentUniverse())) {
        outTextF(ctx, x, y, extent.w, format(_("Cloning %s"),
                                             getCurrentUniverse()->getShip(sid).getName(GObject::PlainName)));
    } else if (int hid = pp.getBaseBuildHull()) {
        outTextF(ctx, x, y, extent.w, format(_("Building a %s"), getHull(hid).getName()));
    } else {
        outTextF(ctx, x, y, extent.w, string_t());
    }
    y += h;

    /* repairing something? */
    if (pp.getBaseShipyardOrder()) {
        outTextF(ctx, x, y, extent.w,
                 format(pp.getBaseShipyardOrder() == 1 ? _("Repairing %s") : _("Recycling %s"),
                        getCurrentUniverse()->getShip(pp.getBaseShipyardId()).getName(GObject::LongName)));
    } else {
        outTextF(ctx, x, y, extent.w, string_t());
    }
    y += h;
    y += h/3;

    /* administrivia */
    outTextF(ctx, x, y, extent.w, format(_("Planetary FCode: %s"), pp.getFCode()));
    y += h;
    outTextF(ctx, x, y, extent.w,
                 format(_("SB Defense: %d; Fighters: %d; Damage: %d%%"),
                        numToString(pp.getStructures(str_BaseDefense)),
                        numToString(pp.getCargo(el_Fighters)),
                        numToString(pp.getBaseDamage())));
    y += h;
}
