/**
  *  \file game/rst.h
  *  \brief Result File Parsing
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_RST_H
#define PCC_V2_GAME_RST_H

#include "game/struct.h"

class Stream;

//! Result File Parser.
class GResultFile {
    TResultHeader header;
    Stream&       file;
    int           version;

    void checkHeader();
 public:
    /** RST Section numbers. The first 8 correspond to the pointers in
        the header. */
    enum Section {
        sec_Ships,              ///< Player's ships.
        sec_Targets,            ///< Visual contacts.
        sec_Planets,            ///< Player's planets.
        sec_Bases,              ///< Player's bases.
        sec_Messages,           ///< Incoming messages.
        sec_ShipXY,             ///< Ship X/Y, nonvisual contacts.
        sec_Gen,                ///< Red tape.
        sec_Vcrs,               ///< Combat recordings.
        sec_Kore,               ///< "KORE" (minefields, race names, storms, contacts, UFOs).
        sec_Leech,              ///< "LEECH.DAT".
        sec_Skore,              ///< "SKORE" (more UFOs).
        sec_LAST = sec_Skore
    };

    GResultFile(Stream& file);
    ~GResultFile();

    /** Get turn file version. This can be -1 (old-style turn, 3.0), 0
        (3.500) or 1 (3.501). No known host generates higher versions,
        but we would support them in theory. */
    int getVersion() const
        { return version; }

    int32_t getSectionOffset(Section section) const;

    bool haveSection(Section section) const;

    /** Get underlying file. */
    Stream& getFile() const
        { return file; }
};

/** Destructor. */
inline GResultFile::~GResultFile()
{ }


#endif
