/**
  *  \file game/utilread.h
  *  \brief UTILx.DAT Reading
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_UTILREAD_H
#define PCC_V2_GAME_UTILREAD_H

#include "cpluslib/object.h"
#include "game/timestamp.h"

class Stream;

//! Logic for reading UTILx.DAT
class GUtilReader : public Object {
 public:
    GUtilReader();
    virtual ~GUtilReader();

    void  read(Stream& s);
    bool  check(Stream& s, GTimestamp* ts);

    /** Process one record. Can throw an exception to abort parsing.
        \param rec_id  record id
        \param size    size of record, in bytes
        \param data    "raw" data
        \pre size != 0 => data != 0
        \return true to continue parsing, false to stop.
        \default returns true */
    virtual bool process(int rec_id, std::size_t size, const char data[]) = 0;

    /** Error report. Reports a file format error (i.e. file too short). */
    virtual void failure(Stream& s) = 0;
};

#endif
