/**
  *  \file int/if/ufoif.h
  *  \brief Interpreter Game Interface: Ufo Properties
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_IF_UFOIF_H
#define PCC_V2_INT_IF_UFOIF_H

#include "int/context.h"
#include "cpluslib/types.h"
#include "game/ufo.h"

class IntValue;
class IntArgBlock;

IntValue*   IFUfoGet(IntArgBlock& args);
IntContext* IFUfoMake();
int32_t     IFUfoDim(int32_t dim);

/** Ufo context. */
class IntUfoContext : public IntContext {
    GUfoType& con;
    GUfoType::iterator it;
 public:
    IntUfoContext(GUfoType& con, GUfoType::iterator it);
    ~IntUfoContext();

    // IntContext:
    int32_t   lookup(const IntNameQuery& name);
    void      set(int32_t index, IntValue* value);
    IntValue* get(int32_t index);
    bool      next();
    IntContext* clone();
    GObject*  getObject();
    string_t  toString(bool readable);
    void      store(IntTagNode& sv, Stream& aux);
    void      enumProperties(IntPropertyAcceptor& acceptor);
};

enum IntUfoProperty {
    iupColorEGA,
    iupColorPCC,
    iupHeadingInt,
    iupHeadingName,
    iupId,
    iupId2,
    iupInfo1,
    iupInfo2,
    iupKeepFlag,
    iupLastScan,
    iupLocX,
    iupLocY,
    iupMarked,
    iupMoveDX,
    iupMoveDY,
    iupName,
    iupRadius,
    iupSpeedInt,
    iupSpeedName,
    iupType,
    iupVisiblePlanet,
    iupVisibleShip
};

IntValue* getUfoProperty(const GUfo& ufo, IntUfoProperty iup);
void      setUfoProperty(GUfo& ufo, IntUfoProperty iup, IntValue* value);

#endif
