/**
  *  \file int/structure.h
  *  \brief Interpreter: Structures
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_STRUCTURE_H
#define PCC_V2_INT_STRUCTURE_H

#include "int/names.h"
#include "int/data.h"
#include "int/context.h"
#include "int/value.h"
#include "cpluslib/smartptr.h"

/** Structure type. Contains everything that makes up a structure type.
    Actually, a structure type is just the member name/slot mapping, so
    this could have been a typedef to IntVariableNames, but making this
    a type keeps the door open for future expansion. */
class IntStructureTypeData {
 public:
    IntStructureTypeData();
    ~IntStructureTypeData();

    IntVariableNames names;
};

/** Handle to a structure type. */
class IntStructureType : public IntValue {
 public:
    IntStructureType(Ptr<IntStructureTypeData> type);
    ~IntStructureType();

    // IntValue:
    string_t toString(bool readable);
    void store(IntTagNode& sv, Stream& aux);
    IntStructureType* clone();

    Ptr<IntStructureTypeData> getType() const
        { return type; }

 private:
    Ptr<IntStructureTypeData> type;
};


/** Structure value. */
class IntStructureValueData {
 public:
    IntStructureValueData(Ptr<IntStructureTypeData> type);
    ~IntStructureValueData();

    Ptr<IntStructureTypeData> type;
    IntDataSegment data;
};

/** Handle to a structure value. */
class IntStructureValue : public IntSingleContext {
 public:
    IntStructureValue(Ptr<IntStructureValueData> value);
    ~IntStructureValue();

    // IntValue:
    string_t toString(bool readable);
    void store(IntTagNode& sv, Stream& aux);
    IntStructureValue* clone();

    // IntContext:
    int32_t lookup(const IntNameQuery& name);
    void set(int32_t index, IntValue* value);
    IntValue* get(int32_t index);
    GObject* getObject();
    void enumProperties(IntPropertyAcceptor& acceptor);

    Ptr<IntStructureValueData> getValue() const
        { return value; }

 private:
    Ptr<IntStructureValueData> value;
};

#endif
