/**
  *  \file u/t_int.h
  *  \brief Testsuite for interpreter (int/).
  *  \ingroup Testsuite
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_U_T_INT_H
#define PCC_V2_U_T_INT_H

#include <cxxtest/TestSuite.h>
#include "u/helper.h"
#include "int/context.h"
#include "int/interror.h"

extern int32_t globalVars[3];

class GlobalTestVars : public IntSingleContext {
 public:
    int32_t lookup(const IntNameQuery& name);
    IntValue* get(int32_t index);
    void set(int32_t index, IntValue* v);
    IntSingleContext* clone();
    GObject* getObject()
        { return 0; }
    string_t toString(bool /*readable*/)
        { return "#<test>"; }
    void store(IntTagNode& /*sv*/, Stream& /*aux*/)
        { throw IntError::notSerializable(); }
    void enumProperties(IntPropertyAcceptor& /*acceptor*/)
        { }
};

void checkIntExpr(const char* expr, int result);
void checkBoolExpr(const char* expr, int result);
void checkFileExpr(const char* expr, int result);
void checkNullExpr(const char* expr);
void checkStringExpr(const char* expr, const char* result);
void checkFloatExpr(const char* expr, double result);
void checkExecFailure(const char* expr);

class IntBuiltinTestSuite : public CxxTest::TestSuite {
 public:
    void testTrig();
    void testAbs();
    void testAsc();
    void testBitOps();
    void testMinMax();
    void testChr();
    void testTypeChecks();
    void testExp();
    void testStrFind();
    void testSubstr();
    void testTrim();
    void testSqrt();
    void testRound();
    void testIf();
    void testStr();
    void testVal();
    void testZap();
    void testLen();
    void testStrMult();
    void testStrCase();
    void testAtom();
    void testEval();
};

class IntCompoundTestSuite : public CxxTest::TestSuite {
 public:
    void testClosure();
};

class IntKeymapTestSuite : public CxxTest::TestSuite {
 public:
    void testParse();
    void testFormat();
    void testKeymap();
};

class IntValueTestSuite : public CxxTest::TestSuite {
 public:
    void testReal();
    void testInt();
    void testString();
};

class IntDataTestSuite : public CxxTest::TestSuite {
 public:
    void testLoad();
};

class IntLexTestSuite : public CxxTest::TestSuite {
 public:
    void testTokenizer();
    void testIntegers();
    void testFloats();
    void testStrings();
};

class IntNotifyTestSuite : public CxxTest::TestSuite {
 public:
    void testNotify();
};

class IntParseExprTestSuite : public CxxTest::TestSuite {
 public:
    void setUp();
    void tearDown();

    void testLiterals();
    void testSequence();
    void testAssignment();
    void testOr();
    void testAnd();
    void testXor();
    void testNot();
    void testComparison();
    void testConcat();
    void testAdd();
    void testSubtract();
    void testMultiply();
    void testDivide();
    void testIntegerDivide();
    void testNegation();
    void testPower();

    void testPrecedence();
};

class IntStatementTestSuite : public CxxTest::TestSuite {
 public:
    void setUp();
    void tearDown();

    void testExprStatement();
    void testIf();
    void testFor();
    void testDo();
    void testSelect();
    void testEval();
};

#endif
